@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <libxml/parser.h>
+#include <time.h>
 
 #include "glade-parser.h"
 #include "glade.h"
@@ -125,9 +126,9 @@ create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
         if (!xmlStrcmp(attrs[i], BAD_CAST("class")))
-            info->classname = alloc_string (interface, CAST_BAD(attrs[i+1]));
+            info->classname = glade_xml_alloc_string (interface, CAST_BAD(attrs[i+1]));
             else if (!xmlStrcmp(attrs[i], BAD_CAST("id")))
-                info->name = alloc_string (interface, CAST_BAD(attrs[i+1]));
+                info->name = glade_xml_alloc_string (interface, CAST_BAD(attrs[i+1]));
             else
                 g_warning("unknown attribute `%s' for <widget>.", attrs[i]);
     }
@@ -233,9 +234,9 @@ handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("action_name")))
-	    info.action_name = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.action_name = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("description")))
-	    info.description = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.description = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <action>.", attrs[i]);
     }
@@ -259,9 +260,9 @@ handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("target")))
-	    info.target = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.target = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("type")))
-	    info.type = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.type = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <signal>.", attrs[i]);
     }
@@ -286,15 +287,15 @@ handle_signal(GladeParseState *state, const xmlChar **attrs)
     info.after = FALSE;
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-	    info.name = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.name = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("handler")))
-	    info.handler = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.handler = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("after")))
 	    info.after = attrs[i+1][0] == 'y';
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("lookup")))
 	    info.lookup = attrs[i+1][0] == 'y';
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("object")))
-	    info.object = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.object = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else if (!xmlStrcmp(attrs[i], BAD_CAST("last_modification_time")))
 	    /* Do nothing. */;
 	else
@@ -378,7 +379,7 @@ handle_accel(GladeParseState *state, const xmlChar **attrs)
                } else
                    pos++;
 	} else if (!xmlStrcmp(attrs[i], BAD_CAST("signal")))
-	    info.signal = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info.signal = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <accelerator>.", attrs[i]);
     }
@@ -416,7 +417,7 @@ handle_child(GladeParseState *state, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!xmlStrcmp(attrs[i], BAD_CAST("internal-child")))
-	    info->internal_child = alloc_string(state->interface, CAST_BAD(attrs[i+1]));
+	    info->internal_child = glade_xml_alloc_string(state->interface, CAST_BAD(attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <child>.", attrs[i]);
     }
@@ -433,7 +434,7 @@ glade_parser_start_document(GladeParseState *state)
     state->widget_depth = 0;
     state->content = g_string_sized_new(128);
 
-    state->interface = glade_interface_new ();
+    state->interface = glade_parser_interface_new ();
     state->widget = NULL;
 
     state->prop_type = PROP_NONE;
@@ -457,6 +458,13 @@ glade_parser_end_document(GladeParseState *state)
 	g_warning("widget_depth != 0 (%d)", state->widget_depth);
 }
 
+static void
+glade_parser_comment (GladeParseState *state, const xmlChar *comment)
+{
+	if (state->state == PARSER_START)
+		state->interface->comment = g_strdup (CAST_BAD (comment));
+}
+
 static void
 glade_parser_start_element(GladeParseState *state,
 			   const xmlChar *name, const xmlChar **attrs)
@@ -499,7 +507,7 @@ glade_parser_start_element(GladeParseState *state,
 		    iface->requires = g_renew(gchar *, iface->requires,
 					      iface->n_requires);
 		    iface->requires[iface->n_requires-1] =
-			alloc_string(iface, CAST_BAD(attrs[i+1]));
+			glade_xml_alloc_string(iface, CAST_BAD(attrs[i+1]));
 		} else
 		    g_warning("unknown attribute `%s' for <requires>.",
 			      attrs[i]);
@@ -547,14 +555,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
@@ -608,14 +616,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
@@ -764,14 +772,14 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!xmlStrcmp(attrs[i], BAD_CAST("name")))
-		    state->prop_name = alloc_propname(state->interface,
+		    state->prop_name = glade_xml_alloc_propname(state->interface,
 						      CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("translatable")))
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
-		    state->comment = alloc_propname(state->interface, 
+		    state->comment = glade_xml_alloc_propname(state->interface, 
 						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
@@ -890,7 +898,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -912,7 +920,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -976,7 +984,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	prop.has_context  = state->context_prop;
 	prop.translatable = state->translate_prop;
 	prop.comment      = state->comment;
-	prop.value        = alloc_string(state->interface, state->content->str);
+	prop.value        = glade_xml_alloc_string(state->interface, state->content->str);
 
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
@@ -1082,7 +1090,7 @@ static xmlSAXHandler glade_parser = {
     (charactersSAXFunc)glade_parser_characters, /* characters */
     0, /* ignorableWhitespace */
     0, /* processingInstruction */
-    (commentSAXFunc)0, /* comment */
+    (commentSAXFunc)glade_parser_comment, /* comment */
     (warningSAXFunc)glade_parser_warning, /* warning */
     (errorSAXFunc)glade_parser_error, /* error */
     (fatalErrorSAXFunc)glade_parser_fatal_error, /* fatalError */
@@ -1109,12 +1117,12 @@ widget_info_free(GladeWidgetInfo *info)
 }
 
 /**
- * glade_interface_new
+ * glade_parser_interface_new
  *
  * Returns a newly allocated GladeInterface.
  */
 GladeInterface *
-glade_interface_new ()
+glade_parser_interface_new ()
 {
 	GladeInterface *interface;
 	interface = g_new0 (GladeInterface, 1);
@@ -1127,13 +1135,13 @@ glade_interface_new ()
 }
 
 /**
- * glade_interface_destroy
+ * glade_parser_interface_destroy
  * @interface: the GladeInterface structure.
  *
  * Frees a GladeInterface structure.
  */
 void
-glade_interface_destroy(GladeInterface *interface)
+glade_parser_interface_destroy (GladeInterface *interface)
 {
     gint i;
 
@@ -1152,11 +1160,13 @@ glade_interface_destroy(GladeInterface *interface)
      * of the strings. */
     g_hash_table_destroy(interface->strings);
 
+    g_free (interface->comment);
+    
     g_free(interface);
 }
 
 /**
- * glade_parser_parse_file
+ * glade_parser_interface_new_from_file
  * @file: the filename of the glade XML file.
  * @domain: the translation domain for the XML file.
  *
@@ -1170,7 +1180,7 @@ glade_interface_destroy(GladeInterface *interface)
  * Returns: the GladeInterface structure for the XML file.
  */
 GladeInterface *
-glade_parser_parse_file(const gchar *file, const gchar *domain)
+glade_parser_interface_new_from_file (const gchar *file, const gchar *domain)
 {
     GladeParseState state = { 0 };
 
@@ -1192,7 +1202,7 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 			       GLADE_UI_ERROR,
 			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
-	    glade_interface_destroy (state.interface);
+	    glade_parser_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
@@ -1200,14 +1210,14 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 			       GLADE_UI_ERROR,
 			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
-	    glade_interface_destroy(state.interface);
+	    glade_parser_interface_destroy(state.interface);
 	return NULL;
     }
     return state.interface;
 }
 
 /**
- * glade_parser_parse_buffer
+ * glade_parser_interface_new_from_buffer
  * @buffer: a buffer in memory containing XML data.
  * @len: the length of @buffer.
  * @domain: the translation domain for the XML file.
@@ -1222,7 +1232,9 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
  * Returns: the GladeInterface structure for the XML buffer.
  */
 GladeInterface *
-glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
+glade_parser_interface_new_from_buffer (const gchar *buffer,
+					gint len,
+					const gchar *domain)
 {
     GladeParseState state = { 0 };
 
@@ -1235,18 +1247,114 @@ glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
     if (xmlSAXUserParseMemory(&glade_parser, &state, buffer, len) < 0) {
 	g_warning("document not well formed!");
 	if (state.interface)
-	    glade_interface_destroy (state.interface);
+	    glade_parser_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
 	g_warning("did not finish in PARSER_FINISH state!");
 	if (state.interface)
-	    glade_interface_destroy(state.interface);
+	    glade_parser_interface_destroy(state.interface);
 	return NULL;
     }
     return state.interface;
 }
 
+static gchar *
+modifier_string_from_bits (GdkModifierType modifiers)
+{
+    GString *string = g_string_new ("");
+
+    if (modifiers & GDK_SHIFT_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_SHIFT_MASK");
+    }
+
+    if (modifiers & GDK_LOCK_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_LOCK_MASK");
+    }
+
+    if (modifiers & GDK_CONTROL_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_CONTROL_MASK");
+    }
+
+    if (modifiers & GDK_MOD1_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD1_MASK");
+    }
+
+    if (modifiers & GDK_MOD2_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD2_MASK");
+    }
+
+    if (modifiers & GDK_MOD3_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD3_MASK");
+    }
+
+    if (modifiers & GDK_MOD4_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD4_MASK");
+    }
+
+    if (modifiers & GDK_MOD5_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_MOD5_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON1_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON1_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON2_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON2_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON3_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON3_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON4_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON4_MASK");
+    }
+
+    if (modifiers & GDK_BUTTON5_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_BUTTON5_MASK");
+    }
+
+    if (modifiers & GDK_RELEASE_MASK) {
+	if (string->len > 0)
+	    g_string_append (string, " | ");
+	g_string_append (string, "GDK_RELEASE_MASK");
+    }
+
+    if (string->len > 0)
+	return g_string_free (string, FALSE);
+
+    g_string_free (string, TRUE);
+    return NULL;
+}
+
 static void
 dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 {
@@ -1369,16 +1477,22 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 
     for (i = 0; i < info->n_accels; i++) {
 	xmlNode *node;
+	gchar   *modifiers;
+
+	modifiers = modifier_string_from_bits (info->accels[i].modifiers);
 
 	for (j = 0; j < indent + 1; j++)
 	    xmlNodeAddContent(widget, BAD_CAST("  "));
 
 	node = xmlNewNode(NULL, BAD_CAST("accelerator"));
 	xmlSetProp(node, BAD_CAST("key"), BAD_CAST(gdk_keyval_name(info->accels[i].key)));
-	xmlSetProp(node, BAD_CAST("modifier"), BAD_CAST("something")/*info->accels[i].modifiers*/);
+	xmlSetProp(node, BAD_CAST("modifiers"), BAD_CAST(modifiers));
 	xmlSetProp(node, BAD_CAST("signal"), BAD_CAST(info->accels[i].signal));
 	xmlAddChild(widget, node);
 	xmlNodeAddContent(widget, BAD_CAST("\n"));
+
+	if (modifiers)
+	    g_free (modifiers);
     }
 
     for (i = 0; i < info->n_children; i++) {
@@ -1446,13 +1560,20 @@ static xmlDoc *
 glade_interface_make_doc (GladeInterface *interface)
 {
     xmlDoc *doc;
-    xmlNode *root;
+    xmlNode *root, *comment;
     gint i;
 
     doc = xmlNewDoc(BAD_CAST("1.0"));
     doc->standalone = FALSE;
     xmlCreateIntSubset(doc, BAD_CAST("glade-interface"),
 		       NULL, BAD_CAST("glade-2.0.dtd"));
+
+    if (interface->comment)
+    {
+	comment = xmlNewComment(BAD_CAST (interface->comment));
+	xmlDocSetRootElement(doc, comment);
+    }
+	
     root = xmlNewNode(NULL, BAD_CAST("glade-interface"));
     xmlDocSetRootElement(doc, root);
 
@@ -1493,7 +1614,7 @@ glade_interface_buffer (GladeInterface  *interface,
 }
 
 /**
- * glade_interface_dump_full
+ * glade_parser_interface_dump
  * @interface: the GladeInterface
  * @filename: the filename to write the interface data to.
  * @error: a #GError for error handleing.
@@ -1504,7 +1625,9 @@ glade_interface_buffer (GladeInterface  *interface,
  * Returns whether the write was successfull or not.
  */
 gboolean
-glade_interface_dump_full(GladeInterface *interface, const gchar *filename, GError **error)
+glade_parser_interface_dump (GladeInterface *interface,
+			     const gchar *filename,
+			     GError **error)
 {
 	GIOChannel *fd;
 	gpointer buffer;
@@ -1531,21 +1654,6 @@ glade_interface_dump_full(GladeInterface *interface, const gchar *filename, GErr
 	return (retval == G_IO_STATUS_NORMAL) ? TRUE : FALSE;
 }
 
-/**
- * glade_interface_dump
- * @interface: the GladeInterface
- * @filename: the filename to write the interface data to.
- *
- * This function dumps the contents of a GladeInterface into a file as
- * XML.  It was originaly intended as a debugging tool.
- */
-void
-glade_interface_dump(GladeInterface *interface, const gchar *filename)
-{
-	glade_interface_dump_full(interface, filename, NULL);
-}
-
-
 G_CONST_RETURN gchar *
 glade_parser_pvalue_from_winfo (GladeWidgetInfo *winfo,
 				const gchar     *pname)
