@@ -44,11 +44,18 @@
 #include "glade-signal.h"
 #include "glade-parameter.h"
 #include "glade-debug.h"
+#include "glade-fixed.h"
 
 /* hash table that will contain all the GtkWidgetClass'es created, indexed by its name */
 static GHashTable *widget_classes = NULL;
 
-#define GLADE_ICON_SIZE 24
+
+#define GLADE_LARGE_ICON_SUBDIR "22x22"
+#define GLADE_LARGE_ICON_SIZE 22
+
+#define GLADE_SMALL_ICON_SUBDIR "16x16"
+#define GLADE_SMALL_ICON_SIZE 16
+
 
 typedef struct {
 	gchar *parent_name;
@@ -95,6 +102,8 @@ glade_widget_class_free (GladeWidgetClass *widget_class)
 	g_free (widget_class->generic_name);
 	g_free (widget_class->name);
 	g_free (widget_class->catalog);
+	if (widget_class->book)
+		g_free (widget_class->book);
 
 	g_list_foreach (widget_class->properties, (GFunc) glade_property_class_free, NULL);
 	g_list_free (widget_class->properties);
@@ -111,35 +120,31 @@ glade_widget_class_free (GladeWidgetClass *widget_class)
 	g_list_foreach (widget_class->signals, (GFunc) glade_signal_free, NULL);
 	g_list_free (widget_class->signals);
 
-	if (widget_class->icon)
-		g_object_unref (G_OBJECT (widget_class->icon));
+	if (widget_class->cursor != NULL)
+		gdk_cursor_unref (widget_class->cursor);
+
+	if (widget_class->large_icon != NULL)
+		g_object_unref (G_OBJECT (widget_class->large_icon));
+
+	if (widget_class->small_icon != NULL)
+		g_object_unref (G_OBJECT (widget_class->small_icon));
+
 }
 
 static GList *
-gwc_props_from_pspecs (GParamSpec **specs, gint n_specs)
+gwc_props_from_pspecs (GladeWidgetClass  *class,
+		       GParamSpec       **specs,
+		       gint               n_specs)
 {
 	GladePropertyClass *property_class;
-	GParamSpec         *spec;
 	gint                i;
 	GList              *list = NULL;
 
 	for (i = 0; i < n_specs; i++)
 	{
-		spec = specs[i];
-
-		/* We only use the writable properties */
-		if (spec->flags & G_PARAM_WRITABLE)
-		{
-			property_class = glade_property_class_new_from_spec (spec);
-			if (!property_class)
-				continue;
-
-			/* Flag the construct only properties */
-			if (spec->flags & G_PARAM_CONSTRUCT_ONLY)
-				property_class->construct_only = TRUE;
-			
+		if ((property_class = 
+		     glade_property_class_new_from_spec (class, specs[i])) != NULL)
 			list = g_list_prepend (list, property_class);
-		}
 	}
 	return g_list_reverse (list);
 }
@@ -150,23 +155,70 @@ glade_widget_class_list_properties (GladeWidgetClass *class)
 	GObjectClass  *object_class;
 	GParamSpec   **specs = NULL;
 	guint          n_specs = 0;
-	GList         *list;
+	GList         *list, *atk_list = NULL;
 
 	g_return_val_if_fail (GLADE_IS_WIDGET_CLASS (class), NULL);
 
 	/* Let it leek */
 	if ((object_class = g_type_class_ref (class->type)) == NULL)
 	{
-		g_warning ("Failed to get class for type %s\n", 
-			   g_type_name (class->type));
+		g_critical ("Failed to get class for type %s\n", 
+			    g_type_name (class->type));
 		return NULL;
 	}
 
+	/* list class properties */
 	specs = g_object_class_list_properties (object_class, &n_specs);
-	list = gwc_props_from_pspecs (specs, n_specs);
+	list = gwc_props_from_pspecs (class, specs, n_specs);
 	g_free (specs);
 
-	return list;
+	/* GtkWidgetClass & descendants have accelerators */
+	if (g_type_is_a (class->type, GTK_TYPE_WIDGET))
+		list = g_list_append (list, glade_property_class_accel_property
+				      (class, class->type));
+
+	/* list the (hard-coded) atk relation properties if applicable */
+	if (glade_util_class_implements_interface (class->type, 
+						   ATK_TYPE_IMPLEMENTOR))
+		atk_list = glade_property_class_list_atk_relations (class, class->type);
+
+	return g_list_concat (list, atk_list);
+}
+
+/*
+This function assignes "weight" to each property in its natural order staring from 1.
+If @parent is 0 weight will be set for every GladePropertyClass in the list.
+This function will not override weight if it is already set (weight >= 0.0)
+*/
+static void
+glade_widget_class_properties_set_weight (GList **properties, GType parent)
+{
+	gint normal = 0, common = 0, packing = 0;
+	GList *l;
+
+	for (l = *properties; l && l->data; l = g_list_next (l))
+	{
+		GladePropertyClass *class = l->data;
+		GPCType type = class->type;
+	
+		if (class->visible &&
+		    (parent) ? parent == class->pspec->owner_type : TRUE &&
+	    	    (type == GPC_NORMAL || type == GPC_ACCEL_PROPERTY))
+		{
+			/* Use a different counter for each tab (common, packing and normal) */
+			if (class->common) common++;
+			else if (class->packing) packing++;
+			else normal++;
+
+			/* Skip if it is already set */
+			if (class->weight >= 0.0) continue;
+			
+			/* Special-casing weight of properties for seperate tabs */
+			if (class->common) class->weight = common;
+			else if (class->packing) class->weight = packing;
+			else class->weight = normal;
+		}
+	}
 }
 
 static GList * 
@@ -193,15 +245,25 @@ glade_widget_class_list_child_properties (GladeWidgetClass *class)
 	}
 
 	specs = gtk_container_class_list_child_properties (object_class, &n_specs);
-	list  = gwc_props_from_pspecs (specs, n_specs);
+	list  = gwc_props_from_pspecs (class, specs, n_specs);
 	g_free (specs);
 
-	/* Mark packing props */
+	/* We have to mark packing properties from GladeWidgetClass
+	 * because GladePropertyClass doesnt have a valid parent GType
+	 * to introspect it.
+	 *
+	 * (which could be used to call gtk_container_class_find_child_property()
+	 * and properly introspect whether or not its a packing property).
+	 */
 	for (l = list; l; l = l->next)
 	{
 		property_class = l->data;
 		property_class->packing = TRUE;
 	}
+	
+	/* Set default weight on packing properties */
+	glade_widget_class_properties_set_weight (&list, 0);
+	
 	return list;
 }
 
@@ -223,8 +285,7 @@ glade_widget_class_list_children (GladeWidgetClass *class)
 		{
 			child->add              = (GladeAddChildFunc)    gtk_container_add;
 			child->remove           = (GladeRemoveChildFunc) gtk_container_remove;
-			child->get_children     = (GladeGetChildrenFunc) gtk_container_get_children;
-			child->get_all_children =
+			child->get_children     =
 				(GladeGetChildrenFunc) glade_util_container_get_all_children;
 			child->set_property     =
 				(GladeChildSetPropertyFunc) gtk_container_child_set_property;
@@ -237,80 +298,197 @@ glade_widget_class_list_children (GladeWidgetClass *class)
 	return children;
 }
 
+static gint
+glade_widget_class_signal_comp (gconstpointer a, gconstpointer b)
+{
+	const GladeSignalClass *signal_a = a, *signal_b = b;	
+	return strcmp (signal_b->query.signal_name, signal_a->query.signal_name);
+}
+
+static void
+glade_widget_class_add_signals (GList **signals, GType type)
+{
+	guint count, *sig_ids, num_signals;
+	GladeSignalClass *cur;
+	GList *list = NULL;
+	
+	if (G_TYPE_IS_INSTANTIATABLE (type) || G_TYPE_IS_INTERFACE (type))
+	{
+		sig_ids = g_signal_list_ids (type, &num_signals);
+
+		for (count = 0; count < num_signals; count++)
+		{
+			cur = g_new0 (GladeSignalClass, 1);
+			
+			g_signal_query (sig_ids[count], &(cur->query));
+
+			/* Since glib gave us this signal id... it should
+			 * exist no matter what.
+			 */
+			g_assert (cur->query.signal_id != 0);
+
+			cur->name = (cur->query.signal_name);
+			cur->type = (gchar *) g_type_name (type);
+
+			list = g_list_prepend (list, cur);
+		}
+		g_free (sig_ids);
+		
+		list = g_list_sort (list, glade_widget_class_signal_comp);
+		*signals = g_list_concat (list, *signals);
+	}
+}
+
 static GList * 
 glade_widget_class_list_signals (GladeWidgetClass *class) 
 {
 	GList *signals = NULL;
-	GType type;
-	guint count;
-	guint *sig_ids;
-	guint num_signals;
-	GladeWidgetClassSignal *cur;
+	GType type, parent, *i, *p;
 
 	g_return_val_if_fail (class->type != 0, NULL);
 
-	type = class->type;
-	while (g_type_is_a (type, G_TYPE_OBJECT))
+	for (type = class->type; g_type_is_a (type, G_TYPE_OBJECT); type = parent)
 	{
-		if (G_TYPE_IS_INSTANTIATABLE (type) || G_TYPE_IS_INTERFACE (type))
-		{
-			sig_ids = g_signal_list_ids (type, &num_signals);
+		parent = g_type_parent (type);
+		
+		/* Add class signals */
+		glade_widget_class_add_signals (&signals, type);
+	
+		/* Add class interfaces signals */
+		for (i = p = g_type_interfaces (type, NULL); *i; i++)
+			if (!glade_util_class_implements_interface (parent, *i))
+				glade_widget_class_add_signals (&signals, *i);
 
-			for (count = 0; count < num_signals; count++)
-			{
-				cur = g_new0 (GladeWidgetClassSignal, 1);
-				cur->name = (gchar *) g_signal_name (sig_ids[count]);
-				cur->type = (gchar *) g_type_name (type);
-
-				signals = g_list_append (signals, (GladeWidgetClassSignal *) cur);
-			}
-			g_free (sig_ids);
-		}
-
-		type = g_type_parent (type);
+		g_free (p);
 	}
 
-	return signals;
+	return g_list_reverse (signals);
 }
 
-static GdkPixbuf *
-glade_widget_class_create_icon (GladeWidgetClass *class)
+static void
+glade_widget_class_load_icons (GladeWidgetClass *class)
 {
-	GdkPixbuf *icon  = NULL;
 	GError    *error = NULL;
 	gchar     *icon_path;
 
-	if (class->generic_name)
+
+	/* only certain widget classes need to have icons */
+	if (G_TYPE_IS_INSTANTIATABLE (class->type) == FALSE ||
+            G_TYPE_IS_ABSTRACT (class->type) != FALSE ||
+            class->generic_name == NULL)
 	{
-		icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s.png", 
-					     glade_pixmaps_dir, 
-
-					     class->generic_name);
-		if ((icon = 
-		     gdk_pixbuf_new_from_file (icon_path, &error)) == NULL)
-		{
-			g_warning ("Unable to load icon %s (%s)",
-				   class->name, error->message);
-			error = (g_error_free (error), NULL);
-
-			/* Load missing image to the stock
-			 */
-			if ((icon = gtk_icon_theme_load_icon 
-			     (gtk_icon_theme_get_default (), 
-			      GTK_STOCK_MISSING_IMAGE, 
-			      GLADE_ICON_SIZE, 
-			      GTK_ICON_LOOKUP_USE_BUILTIN, &error)) == NULL)
-			{
-				g_critical ("Unable to load an icon for "
-					    "%s from stock (%s)", 
-					    class->name, 
-					    error->message);
-				error = (g_error_free (error), NULL);
-			}
-		}
-		g_free (icon_path);
+		return;
 	}
-	return icon;
+
+	/* load large 22x22 icon */
+	icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S GLADE_LARGE_ICON_SUBDIR G_DIR_SEPARATOR_S "%s.png", 
+				     glade_pixmaps_dir, 
+				     class->generic_name);
+
+	class->large_icon = gdk_pixbuf_new_from_file_at_size (icon_path, 
+							      GLADE_LARGE_ICON_SIZE, 
+							      GLADE_LARGE_ICON_SIZE, 
+							      &error);
+	
+	if (class->large_icon == NULL)
+	{
+		g_warning (_("Unable to load icon for %s (%s)"), class->name, error->message);
+
+		g_error_free (error);
+		error = NULL;
+
+		/* use stock missing icon */
+		class->large_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), 
+							      GTK_STOCK_MISSING_IMAGE, 
+						  	      GLADE_LARGE_ICON_SIZE, 
+							      GTK_ICON_LOOKUP_USE_BUILTIN, &error);
+		if (class->large_icon == NULL)
+		{
+			g_critical (_("Unable to load stock icon (%s)"),
+				    error->message);
+
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+	g_free (icon_path);
+
+
+	/* load small 16x16 icon */
+	icon_path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S GLADE_SMALL_ICON_SUBDIR G_DIR_SEPARATOR_S "%s.png", 
+				     glade_pixmaps_dir, 
+				     class->generic_name);
+
+	class->small_icon = gdk_pixbuf_new_from_file_at_size (icon_path, 
+							      GLADE_SMALL_ICON_SIZE, 
+							      GLADE_SMALL_ICON_SIZE, 
+							      &error);
+	
+	if (class->small_icon == NULL)
+	{
+		g_warning (_("Unable to load icon for %s (%s)"), class->name, error->message);
+
+		g_error_free (error);
+		error = NULL;
+
+		/* use stock missing icon */
+		class->small_icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), 
+							      GTK_STOCK_MISSING_IMAGE, 
+						  	      GLADE_SMALL_ICON_SIZE, 
+							      GTK_ICON_LOOKUP_USE_BUILTIN, &error);
+		if (class->small_icon == NULL)
+		{
+			g_critical (_("Unable to load stock icon (%s)"),
+				    error->message);
+
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+
+	g_free (icon_path);
+
+}
+
+static void
+glade_widget_class_create_cursor (GladeWidgetClass *widget_class)
+{
+	GdkPixbuf *tmp_pixbuf;
+	const GdkPixbuf *add_pixbuf;
+	GdkDisplay *display;
+
+	/* only certain widget classes need to have cursors */
+	if (G_TYPE_IS_INSTANTIATABLE (widget_class->type) == FALSE ||
+            G_TYPE_IS_ABSTRACT (widget_class->type) != FALSE ||
+            widget_class->generic_name == NULL)
+		return;
+
+	/* cannot continue if 'add widget' cursor pixbuf has not been loaded for any reason */
+	if ((add_pixbuf = glade_cursor_get_add_widget_pixbuf ()) == NULL)
+		return;
+
+	display = gdk_display_get_default ();
+
+	/* create a temporary pixbuf clear to transparent black*/
+	tmp_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 32, 32);
+	gdk_pixbuf_fill (tmp_pixbuf, 0x00000000);
+
+
+	/* composite pixbufs */
+	gdk_pixbuf_composite (widget_class->large_icon, tmp_pixbuf,
+			      8, 8, 22, 22,
+			      8, 8, 1, 1,
+                              GDK_INTERP_NEAREST, 255);
+
+	gdk_pixbuf_composite (add_pixbuf, tmp_pixbuf,
+			      0, 0, 12, 12,
+			      0, 0, 1, 1,
+                              GDK_INTERP_NEAREST, 255);
+
+
+	widget_class->cursor = gdk_cursor_new_from_pixbuf (display, tmp_pixbuf, 6, 6);
+
+	g_object_unref(tmp_pixbuf);
 }
 
 static void
@@ -337,6 +515,9 @@ glade_widget_class_update_properties_from_node (GladeXmlNode      *node,
 		if (!id)
 			continue;
 
+		/* property names from catalogs also need to have the '-' form */
+		glade_util_replace (id, '_', '-');
+		
 		/* find the property in our list, if not found append a new property */
 		for (list = *properties; list && list->data; list = list->next)
 		{
@@ -352,7 +533,7 @@ glade_widget_class_update_properties_from_node (GladeXmlNode      *node,
 		}
 		else
 		{
-			property_class = glade_property_class_new ();
+			property_class = glade_property_class_new (widget_class);
 			property_class->id = g_strdup (id);
 			*properties = g_list_append (*properties, property_class);
 			list = g_list_last (*properties);
@@ -445,22 +626,6 @@ glade_widget_class_find_child_by_type (GladeSupportedChild *child, GType type)
 	return child->type - type;
 }
 
-static void
-glade_widget_class_load_function (GladeXmlNode     *node,
-				  GModule          *module,
-				  gchar            *tagname,
-				  void            **func_location)
-{
-	gchar *buff;
-	if ((buff = glade_xml_get_value_string (node, tagname)) != NULL)
-	{
-		if (!g_module_symbol (module, buff, func_location))
-			g_warning ("Could not find %s in %s\n",
-				   buff, g_module_name (module));
-		g_free (buff);
-	}
-}
-
 static void
 glade_widget_class_update_children_from_node (GladeXmlNode     *node,
 					      GladeWidgetClass *widget_class,
@@ -518,27 +683,24 @@ glade_widget_class_update_children_from_node (GladeXmlNode     *node,
 
 		if (widget_class->module)
 		{
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_ADD_CHILD_FUNCTION,
-							  (void **)&child->add);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_REMOVE_CHILD_FUNCTION,
-							  (void **)&child->remove);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_GET_CHILDREN_FUNCTION,
-							  (void **)&child->get_children);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_GET_ALL_CHILDREN_FUNCTION,
-							  (void **)&child->get_all_children);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_CHILD_SET_PROP_FUNCTION,
-							  (void **)&child->set_property);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_CHILD_GET_PROP_FUNCTION,
-							  (void **)&child->get_property);
-			glade_widget_class_load_function (child_node, widget_class->module,
-							  GLADE_TAG_REPLACE_CHILD_FUNCTION,
-							  (void **)&child->replace_child);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_ADD_CHILD_FUNCTION,
+						      (gpointer *)&child->add);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_REMOVE_CHILD_FUNCTION,
+						      (gpointer *)&child->remove);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_GET_CHILDREN_FUNCTION,
+						      (gpointer *)&child->get_children);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_CHILD_SET_PROP_FUNCTION,
+						      (gpointer *)&child->set_property);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_CHILD_GET_PROP_FUNCTION,
+						      (gpointer *)&child->get_property);
+			glade_xml_load_sym_from_node (child_node, widget_class->module,
+						      GLADE_TAG_REPLACE_CHILD_FUNCTION,
+						      (gpointer *)&child->replace_child);
 		}
 
 		/* if we found a <Properties> tag on the xml file, we add the 
@@ -568,23 +730,26 @@ glade_widget_class_extend_with_node (GladeWidgetClass *widget_class,
 
 	if (widget_class->module)
 	{
-		glade_widget_class_load_function (node, widget_class->module,
-						  GLADE_TAG_POST_CREATE_FUNCTION,
-						  (void **)&widget_class->post_create_function);
+		glade_xml_load_sym_from_node (node, widget_class->module,
+					      GLADE_TAG_POST_CREATE_FUNCTION,
+					      (void **)&widget_class->post_create_function);
 
-		glade_widget_class_load_function (node, widget_class->module,
-						  GLADE_TAG_GET_INTERNAL_CHILD_FUNCTION,
-						  (void **)&widget_class->get_internal_child);
+		glade_xml_load_sym_from_node (node, widget_class->module,
+					      GLADE_TAG_GET_INTERNAL_CHILD_FUNCTION,
+					      (void **)&widget_class->get_internal_child);
 
-		glade_widget_class_load_function (node, widget_class->module,
-						  GLADE_TAG_GET_INTERNAL_CHILDREN_FUNCTION,
-						  (void **)&widget_class->get_internal_children);
-
-		glade_widget_class_load_function (node, widget_class->module,
-						  GLADE_TAG_LAUNCH_EDITOR_FUNCTION,
-						  (void **)
-						  &widget_class->launch_editor);
+		glade_xml_load_sym_from_node (node, widget_class->module,
+					      GLADE_TAG_LAUNCH_EDITOR_FUNCTION,
+					      (void **)
+					      &widget_class->launch_editor);
 	}
+	widget_class->fixed = 
+		glade_xml_get_property_boolean (node, GLADE_TAG_FIXED, widget_class->fixed);
+
+	/* Check if this class is toplevel */
+	widget_class->toplevel =
+		glade_xml_get_property_boolean (node, GLADE_XML_TAG_TOPLEVEL,
+						widget_class->toplevel);
 
 	/* if we found a <properties> tag on the xml file, we add the properties
 	 * that we read from the xml file to the class.
@@ -802,7 +967,6 @@ glade_widget_class_clone_child (GladeSupportedChild *child,
 	clone->add               = child->add;
 	clone->remove            = child->remove;
 	clone->get_children      = child->get_children;
-	clone->get_all_children  = child->get_all_children;
 	clone->set_property      = child->set_property;
 	clone->get_property      = child->get_property;
 	clone->replace_child     = child->replace_child;
@@ -824,8 +988,6 @@ glade_widget_class_merge_child (GladeSupportedChild *widgets_child,
 		widgets_child->remove           = parents_child->remove;
 	if (!widgets_child->get_children)
 		widgets_child->get_children     = parents_child->get_children;
-	if (!widgets_child->get_all_children)
-		widgets_child->get_all_children = parents_child->get_all_children;
 	if (!widgets_child->set_property)
 		widgets_child->set_property     = parents_child->set_property;
 	if (!widgets_child->get_property)
@@ -893,6 +1055,9 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 	g_return_if_fail (GLADE_IS_WIDGET_CLASS (widget_class));
 	g_return_if_fail (GLADE_IS_WIDGET_CLASS (parent_class));
 
+	if (widget_class->fixed == FALSE)
+		widget_class->fixed = parent_class->fixed;
+	
 	if (widget_class->post_create_function == NULL)
 		widget_class->post_create_function = parent_class->post_create_function;
 
@@ -902,6 +1067,9 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 	if (widget_class->launch_editor == NULL)
 		widget_class->launch_editor = parent_class->launch_editor;
 
+	if (widget_class->toplevel == FALSE)
+		widget_class->toplevel = parent_class->toplevel;
+	
 	/* merge the parent's properties */
 	glade_widget_class_merge_properties
 		(parent_class->type,
@@ -913,59 +1081,23 @@ glade_widget_class_merge (GladeWidgetClass *widget_class,
 }
 
 /**
- * glade_widget_class_load_library:
- * @library_name: name of the library
+ * glade_widget_class_new:
+ * @class_node: A #GladeXmlNode
+ * @catname: the name of the owning catalog
+ * @library: the name of the library used to load class methods from
+ * @domain: the domain to translate strings from this plugin from
+ * @book: the devhelp search domain for the owning catalog.
  *
- * Loads the named library from the Glade modules directory, or failing that
- * from the standard platform specific directories.
- *
- * The @library_name should not include any platform specifix prefix or suffix,
- * those are automatically added, if needed, by g_module_build_path()
- *
- * Returns: a #GModule on success, or %NULL on failure.
+ * Merges the contents of the @parent_class on the @widget_class.
+ * The properties of the @parent_class will be prepended to
+ * those of @widget_class.
  */
-static GModule *
-glade_widget_class_load_library (const gchar *library_name)
-{
-	gchar   *path;
-	GModule *module;
-
-	path = g_module_build_path (glade_modules_dir, library_name);
-	if (!path)
-	{
-		g_warning (_("Not enough memory."));
-		return NULL;
-	}
-
-	module = g_module_open (path, G_MODULE_BIND_LAZY);
-	g_free (path);
-
-	if (!module)
-	{
-		path = g_module_build_path (NULL, library_name);
-		if (!path)
-		{
-			g_warning (_("Not enough memory."));
-			return NULL;
-		}
-
-		module = g_module_open (path, G_MODULE_BIND_LAZY);
-		g_free (path);
-	}
-
-	if (!module)
-	{
-		g_warning (_("Unable to open the module %s."), library_name);
-	}
-
-	return module;
-}
-
 GladeWidgetClass *
 glade_widget_class_new (GladeXmlNode *class_node,
 			const gchar  *catname, 
 			const gchar  *library, 
-			const gchar  *domain)
+			const gchar  *domain,
+			const gchar  *book)
 {
 	GladeWidgetClass *widget_class;
 	gchar            *name, *generic_name, *ptr;
@@ -980,13 +1112,9 @@ glade_widget_class_new (GladeXmlNode *class_node,
 		return NULL;
 	}
 	
-	name = glade_xml_get_property_string (class_node, GLADE_TAG_NAME);
-	if (!name)
-	{
-		g_warning ("Required property 'name' not found in '%s'",
-			   GLADE_TAG_GLADE_WIDGET_CLASS);
+	if ((name = glade_xml_get_property_string_required
+	     (class_node, GLADE_TAG_NAME, NULL)) == NULL)
 		return NULL;
-	}
 
 	if (glade_widget_class_get_by_name (name)) 
 	{
@@ -996,10 +1124,20 @@ glade_widget_class_new (GladeXmlNode *class_node,
 		return NULL;
 	}
 
-	generic_name = glade_xml_get_property_string (class_node, 
-						      GLADE_TAG_GENERIC_NAME);
-	title = glade_xml_get_property_string (class_node, GLADE_TAG_TITLE);
+	/* Perform a stupid fallback as generic_name is not required
+	 * for uninstantiatable classes
+	 */
+	generic_name = glade_xml_get_property_string
+		(class_node, GLADE_TAG_GENERIC_NAME);
 
+	if ((title = glade_xml_get_property_string_required
+	     (class_node, GLADE_TAG_TITLE,
+	      "This value is needed to display object class names in the UI")) == NULL)
+	{
+		g_warning ("Class '%s' built without a '%s'", name, GLADE_TAG_TITLE);
+		title = g_strdup (name);
+	}
+	
 	/* Translate title */
 	if (title != dgettext (domain, title))
 	{
@@ -1011,7 +1149,7 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	module = NULL;
 	if (library) 
 	{
-		module = glade_widget_class_load_library (library);
+		module = glade_util_load_library (library);
 		if (!module)
 		{
 			g_warning ("Failed to load external library '%s'",
@@ -1028,13 +1166,32 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	widget_class->module       = module;
 	widget_class->generic_name = generic_name;
 	widget_class->palette_name = title;
-	widget_class->in_palette   = title ? TRUE : FALSE;
 	widget_class->type         = glade_util_get_type_from_name (name);
+	widget_class->cursor       = NULL;
+	widget_class->large_icon   = NULL;
+	widget_class->small_icon   = NULL;
+	widget_class->toplevel     = FALSE;
 
+	if (G_TYPE_IS_INSTANTIATABLE (widget_class->type)    &&
+	    G_TYPE_IS_ABSTRACT (widget_class->type) == FALSE &&
+	    widget_class->generic_name == NULL)
+	{
+		g_warning ("Instantiatable class '%s' built without a '%s'",
+			   name, GLADE_TAG_GENERIC_NAME);
+	}
+
+	/* Perform a stoopid fallback just incase */
+	if (widget_class->generic_name == NULL)
+		widget_class->generic_name = g_strdup ("widget");
+
+	
 	/* Dont mention gtk+ as a required lib in the generated glade file */
 	if (strcmp (catname, "gtk+"))
 		widget_class->catalog = g_strdup (catname);
 
+	if (book)
+		widget_class->book = g_strdup (book);
+
 	if (widget_class->type == 0)
 	{
 		glade_widget_class_free (widget_class);
@@ -1044,7 +1201,9 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	widget_class->properties = glade_widget_class_list_properties (widget_class);
 	widget_class->signals    = glade_widget_class_list_signals (widget_class);
 	widget_class->children   = glade_widget_class_list_children (widget_class);
-	widget_class->icon       = glade_widget_class_create_icon (widget_class);
+
+	glade_widget_class_load_icons (widget_class);
+	glade_widget_class_create_cursor (widget_class);
 
 	for (parent_type = g_type_parent (widget_class->type);
 	     parent_type != 0;
@@ -1056,13 +1215,11 @@ glade_widget_class_new (GladeXmlNode *class_node,
 		parent_name  = g_type_name (parent_type);
 		parent_class = glade_widget_class_get_by_name (parent_name);
 
-		if (parent_class) {
+		if (parent_class)
 			glade_widget_class_merge (widget_class, parent_class);
-		}
 	}
 	
-	if (glade_xml_node_get_children (class_node))
-		glade_widget_class_extend_with_node (widget_class, class_node, domain);
+	glade_widget_class_extend_with_node (widget_class, class_node, domain);
 	
 	/* store the GladeWidgetClass on the cache,
 	 * if it's the first time we store a widget class, then
@@ -1071,6 +1228,13 @@ glade_widget_class_new (GladeXmlNode *class_node,
 	if (!widget_classes)
 		widget_classes = g_hash_table_new (g_str_hash, g_str_equal);		
 	g_hash_table_insert (widget_classes, widget_class->name, widget_class);
+
+	/* Set default weight on properties */
+	for (parent_type = widget_class->type;
+	     parent_type != 0;
+	     parent_type = g_type_parent (parent_type))
+		glade_widget_class_properties_set_weight (&widget_class->properties,
+							  parent_type);
 	
 	return widget_class;
 }
@@ -1224,6 +1388,84 @@ glade_widget_class_get_child_support  (GladeWidgetClass *class,
 }
 
 
+/**
+ * glade_widget_class_default_params:
+ * @class: a #GladeWidgetClass
+ * @construct: whether to return construct params or not construct params
+ * @n_params: return location if any defaults are specified for this class.
+ * 
+ * Returns: A list of params for use in g_object_newv ()
+ */
+GParameter *
+glade_widget_class_default_params (GladeWidgetClass *class,
+				   gboolean          construct,
+				   guint            *n_params)
+{
+	GArray              *params;
+	GObjectClass        *oclass;
+	GParamSpec         **pspec;
+	GladePropertyClass  *pclass;
+	guint                n_props, i;
+
+	g_return_val_if_fail (GLADE_IS_WIDGET_CLASS (class), NULL);
+	g_return_val_if_fail (n_params != NULL, NULL);
+
+	/* As a slight optimization, we never unref the class
+	 */
+	oclass = g_type_class_ref (class->type);
+	pspec  = g_object_class_list_properties (oclass, &n_props);
+	params = g_array_new (FALSE, FALSE, sizeof (GParameter));
+
+	for (i = 0; i < n_props; i++)
+	{
+		GParameter parameter = { 0, };
+
+		pclass = glade_widget_class_get_property_class
+			(class, pspec[i]->name);
+		
+		/* Ignore properties based on some criteria
+		 */
+		if (pclass == NULL       || /* Unaccounted for in the builder */
+		    pclass->set_function || /* should not be set before 
+					       GladeWidget wrapper exists */
+		    pclass->ignore)         /* Catalog explicitly ignores the object */
+			continue;
+
+		if (construct &&
+		    (pspec[i]->flags & 
+		     (G_PARAM_CONSTRUCT|G_PARAM_CONSTRUCT_ONLY)) == 0)
+			continue;
+		else if (!construct &&
+			 (pspec[i]->flags & 
+			  (G_PARAM_CONSTRUCT|G_PARAM_CONSTRUCT_ONLY)) != 0)
+			continue;
+
+
+		if (g_value_type_compatible (G_VALUE_TYPE (pclass->def),
+					     pspec[i]->value_type) == FALSE)
+		{
+			g_critical ("Type mismatch on %s property of %s",
+				    parameter.name, class->name);
+			continue;
+		}
+
+		if (g_param_values_cmp (pspec[i], 
+					pclass->def, 
+					pclass->orig_def) == 0)
+			continue;
+
+		parameter.name = pspec[i]->name; /* These are not copied/freed */
+		g_value_init (&parameter.value, pspec[i]->value_type);
+		g_value_copy (pclass->def, &parameter.value);
+
+		g_array_append_val (params, parameter);
+	}
+	g_free (pspec);
+
+	*n_params = params->len;
+	return (GParameter *)g_array_free (params, FALSE);
+}
+
 void
 glade_widget_class_container_add (GladeWidgetClass *class,
 				  GObject          *container,
@@ -1297,7 +1539,6 @@ glade_widget_class_container_has_child (GladeWidgetClass *class,
 	return found;
 }
 
-
 GList *
 glade_widget_class_container_get_children (GladeWidgetClass *class,
 					   GObject          *container)
@@ -1308,28 +1549,8 @@ glade_widget_class_container_get_children (GladeWidgetClass *class,
 	{
 		GladeSupportedChild *support = list->data;
 		if (support->get_children)
-			children = g_list_concat (children, support->get_children (container));
-	}
-	return children;
-}
-
-
-GList *
-glade_widget_class_container_get_all_children (GladeWidgetClass *class,
-					       GObject          *container)
-{
-	GList *list, *children = NULL;
-
-
-	if (class->get_internal_children)
-		children = class->get_internal_children (container);
-
-	for (list = class->children; list && list->data; list = list->next)
-	{
-		GladeSupportedChild *support = list->data;
-		if (support->get_all_children)
-			children = g_list_concat (children,
-						  support->get_all_children (container));
+			children = g_list_concat
+				(children, support->get_children (container));
 	}
 
 	/* Remove duplicates */
@@ -1497,3 +1718,154 @@ glade_widget_class_get_packing_default (GladeWidgetClass *child_class,
 
 	return NULL;
 }
+
+/**
+ * glade_widget_class_query:
+ * @class: A #GladeWidgetClass
+ *
+ * Returns: whether the user needs to be queried for
+ * certain properties upon creation of this class.
+ */
+gboolean
+glade_widget_class_query (GladeWidgetClass *class)
+{
+	GladePropertyClass *pclass;
+	GList *l;
+
+	for (l = class->properties; l; l = l->next)
+	{
+		pclass = l->data;
+
+		if (pclass->query)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+/**
+ * glade_widget_class_create_widget:
+ * @class: a #GladeWidgetClass
+ * @query: whether to display query dialogs if
+ *         applicable to the class
+ * @...: a %NULL terminated list of string/value pairs of #GladeWidget 
+ *       properties
+ *
+ *
+ * This factory function returns a new #GladeWidget of the correct type/class
+ * with the properties defined in @... and queries the user if nescisary.
+ *
+ * The resulting object will have all default properties applied to it
+ * including the overrides specified in the catalog, unless the catalog
+ * has specified 'ignore' for that property.
+ *
+ * Note that the widget class must be fed twice; once as the
+ * leading arg... and also as the property for the #GladeWidget
+ *
+ * this macro returns the newly created #GladeWidget
+ */
+GladeWidget *
+glade_widget_class_create_widget_real (gboolean          query, 
+				       const gchar      *first_property,
+				       ...)
+{
+	GladeWidgetClass *widget_class;
+	GType             gwidget_type;
+	GladeWidget      *gwidget;
+	va_list           vl, vl_copy;
+
+	g_return_val_if_fail (strcmp (first_property, "class") == 0, NULL);
+
+	va_start (vl, first_property);
+	va_copy (vl_copy, vl);
+
+	widget_class = va_arg (vl, GladeWidgetClass *);
+
+	va_end (vl);
+
+	if (GLADE_IS_WIDGET_CLASS (widget_class) == FALSE)
+	{
+		g_critical ("No class found in glade_widget_class_create_widget_real args");
+		va_end (vl_copy);
+		return NULL;
+	}
+
+	if (widget_class->fixed)
+		gwidget_type = GLADE_TYPE_FIXED;
+	else 
+		gwidget_type = GLADE_TYPE_WIDGET;
+
+
+	gwidget = (GladeWidget *)g_object_new_valist (gwidget_type,
+						      first_property, 
+						      vl_copy);
+	va_end (vl_copy);
+	
+	if (query && glade_widget_class_query (widget_class))
+	{
+		GladeEditor *editor = glade_app_get_editor ();
+		
+		/* If user pressed cancel on query popup. */
+		if (!glade_editor_query_dialog (editor, gwidget))
+		{
+			g_object_unref (G_OBJECT (gwidget));
+			return NULL;
+		}
+	}
+
+	return gwidget;
+}
+
+
+
+/**
+ * glade_widget_class_create_internal:
+ * @parent:            The parent #GladeWidget, or %NULL for children
+ *                     outside of the hierarchy.
+ * @internal_object:   the #GObject
+ * @internal_name:     a string identifier for this internal widget.
+ * @anarchist:         Whether or not this widget is a widget outside
+ *                     of the parent's hierarchy (like a popup window)
+ * @reason:            The #GladeCreateReason for which this internal widget
+ *                     was created (usually just pass the reason from the post_create
+ *                     function; note also this is used only by the plugin code so
+ *                     pass something usefull here).
+ *
+ * A convenienve function to create a #GladeWidget of the prescribed type
+ * for internal widgets.
+ *
+ * Returns: a freshly created #GladeWidget wrapper object for the
+ *          @internal_object of name @internal_name
+ */
+GladeWidget *
+glade_widget_class_create_internal (GladeWidget      *parent,
+				    GObject          *internal_object,
+				    const gchar      *internal_name,
+				    const gchar      *parent_name,
+				    gboolean          anarchist,
+				    GladeCreateReason reason)
+{
+	GladeWidgetClass *class;
+	GladeProject     *project;
+
+	g_return_val_if_fail (GLADE_IS_WIDGET (parent), NULL);
+	project = glade_widget_get_project (parent);
+
+        if ((class = glade_widget_class_get_by_name 
+	     (G_OBJECT_TYPE_NAME (internal_object))) == NULL)
+	{
+		g_critical ("Unable to find widget class for type %s", 
+			    G_OBJECT_TYPE_NAME (internal_object));
+		return NULL;
+	}
+
+	return glade_widget_class_create_widget (class, FALSE,
+						 "anarchist", anarchist,
+						 "parent", parent,
+						 "project", project,
+						 "internal", internal_name,
+						 "internal-name", parent_name,
+						 "reason", reason,
+						 "object", internal_object,
+						 NULL);
+}
